@@ -14,29 +14,56 @@ FP::AST::Perl -- abstract syntax tree for representing Perl code
 =head1 SYNOPSIS
 
     use FP::AST::Perl ":all";
-    use FP::List;
+    use FP::List; use FP::Equal ":all";
 
     is Get(LexVar('foo'))->string, '$foo';
     is Get(PackVarScalar "foo")->string, '$foo';
     is Get(PackVarArray "foo")->string, '@foo';
     is Get(PackVarHash "foo::bar")->string, '%foo::bar';
 
-    my $foo= PackVarCode("foo");
+    my $codefoo= PackVarCode("foo");
     my $arrayfoo= PackVarArray("foo");
     my $lexfoo= LexVar("foo");
-    is App(Get($foo), list())->string,
+    is App(Get($codefoo), list())->string,
        '&foo()';
-    is AppProto(Get($foo), list())->string,
+    is AppProto(Get($codefoo), list())->string,
        'foo()';
-    is AppProto(Get($lexfoo), list(Ref($foo)))->string,
+    is AppProto(Get($lexfoo), list(Ref($codefoo)))->string,
        '$foo->(\&foo)';
     is AppProto(Get($lexfoo), list(Get($lexfoo), Get($arrayfoo), Ref($arrayfoo)))->string,
        '$foo->($foo, @foo, \@foo)';
-    is AppProto(Get($foo), list(Get(LexVar 'foo'), Literal(Number 123)))->string,
+    is AppProto(Get($codefoo), list(Get(LexVar 'foo'), Literal(Number 123)))->string,
        'foo($foo, 123)';
-    is AppProto(Get($foo), list(Get($lexfoo), Literal(String 123)))->string,
+    is AppProto(Get($codefoo), list(Get($lexfoo), Literal(String 123)))->string,
        'foo($foo, \'123\')';
 
+    # Semicolons are like a compile time (AST level) operator:
+    is Semicolon(Get(LexVar "a"), Get(LexVar "b"))->string, '$a; $b';
+    # to end with a semicolon you could use:
+    is Semicolon(Get(LexVar "a"), Noop)->string, '$a; ';
+
+    # The n-ary `semicolons` function builds a Semicolon chain for
+    # you (right-associated):
+    is_equal semicolons(), Noop;
+    is_equal semicolons("x"), "x";
+        # ^ no `Semicolon` instantiation thus no type failure because
+        #   of the string
+    my $sems= semicolons(map {Get LexVar $_} qw(a b c));
+    is_equal $sems,
+             Semicolon(Get(LexVar('a')),
+                       Semicolon(Get(LexVar('b')),
+                                 Get(LexVar('c'))));
+    is $sems->string, '$a; $b; $c';
+
+    is Let(list(LexVar("foo"), LexVar("bar")),
+           Get(PackVarArray "baz"),
+           semicolons(
+               AppProto(Get(PackVarCode "print"),
+                        list Literal String "Hello"),
+               Get(LexVar("bar"))))->string,
+       'my ($foo, $bar) = @baz; print(\'Hello\'); $bar';
+    # Yes, how should print, map etc. be handled?
+
 
 =head1 DESCRIPTION
 
@@ -62,21 +89,33 @@ or on the L<website|http://functional-perl.org/>.
 package FP::AST::Perl;
 @ISA="Exporter"; require Exporter;
 @EXPORT=qw();
-@EXPORT_OK=qw(
-    is_packvar_type
-    is_var
+my @classes=qw(
     LexVar PackVarScalar PackVarCode PackVarHash PackVarArray PackVarGlob
-    is_lexvar is_packvar
-    is_expr
     App AppProto Get Ref
     Number String
-    Literal);
+    Literal
+    Semicolon Noop Let);
+@EXPORT_OK=(
+    @classes, qw(
+    is_packvar_type
+    is_var
+    is_lexvar is_packvar
+    is_expr is_nonnoop_expr is_noop
+    semicolons));
+
 %EXPORT_TAGS=(all=>[@EXPORT,@EXPORT_OK]);
 
 use strict; use warnings; use warnings FATAL => 'uninitialized';
 use Function::Parameters qw(:strict);
 use FP::Predicates ":all";
 use Chj::TEST;
+use FP::List;
+use FP::Combinators2 qw(right_associate_);
+
+# import the constructors of the classes defined below
+for my $name (@classes) {
+    "FP::AST::Perl::${name}::constructors"->import()
+}
 
 
 package FP::AST::_::Perl {
@@ -197,7 +236,7 @@ package FP::AST::Perl::Expr {
 }
 
 *is_expr= instance_of "FP::AST::Perl::Expr";
-
+*is_nonnoop_expr= both *is_expr, complement *is_noop;
 
 # Do we need to distinguish context (list vs. scalar [vs. void]),
 # really? No, since the *dynamic* context determines this!
@@ -268,12 +307,16 @@ package FP::AST::Perl::App {
     
     use FP::Struct [
         [*FP::AST::Perl::is_expr, 'proc'],
-        [list_of *FP::AST::Perl::is_expr, 'args'],
+        [list_of *FP::AST::Perl::is_expr, 'argexprs'],
+        # ^ yes, proc is also an expr, but only yielding one (usable)
+        # value, as opposed to argexprs which may yield more used
+        # values than there are exprs (at least here; not
+        # (necessarily) in AppProto).
         ] => "FP::AST::Perl::Expr";
 
     method string () {
         $self->proc_string . $self->proc->callderef . "(" .
-            $self->args->map(the_method "string")->strings_join (", ") .
+            $self->argexprs->map(the_method "string")->strings_join (", ") .
             # ^ XX are parens needed around arguments?
             ")"
     }
@@ -302,7 +345,6 @@ package FP::AST::Perl::Value {
     use FP::Predicates ":all";
     
     use FP::Struct [
-        'perlvalue' # native Perl value
         ] => "FP::AST::_::Perl";
 
     # missing string method  XX interface
@@ -314,8 +356,10 @@ package FP::AST::Perl::Value {
 
 package FP::AST::Perl::Number {
     use FP::Predicates ":all";
-    
+    use Scalar::Util qw(looks_like_number);
+
     use FP::Struct [
+        [*looks_like_number, 'perlvalue'],
         ] => "FP::AST::Perl::Value";
 
     method string () {
@@ -330,6 +374,7 @@ package FP::AST::Perl::String {
     use Chj::singlequote;
 
     use FP::Struct [
+        'perlvalue',
         ] => "FP::AST::Perl::Value";
 
     method string () {
@@ -357,14 +402,58 @@ package FP::AST::Perl::Literal {
 }
 
 
+package FP::AST::Perl::Semicolon {
+    use FP::Struct [
+        [*FP::AST::Perl::is_expr, 'a'],
+        [*FP::AST::Perl::is_expr, 'b'],
+        ] => "FP::AST::Perl::Expr";
+
+    method string () {
+        $self->a->string . "; " . $self->b->string
+    }
+    _END_
+}
+
+package FP::AST::Perl::Noop {
+    use FP::Struct [
+        ] => "FP::AST::Perl::Expr";
+
+    method string () {
+        ""
+    }
+    _END_
+}
+
+*is_noop= instance_of "FP::AST::Perl::Noop";
 
 
-for my $name (qw(
-              LexVar PackVarScalar PackVarCode PackVarHash PackVarArray PackVarGlob
-              App AppProto Get Ref
-              Number String
-              Literal)) {
-    "FP::AST::Perl::${name}::constructors"->import()
+*semicolons= right_associate_ *Semicolon, Noop();
+
+
+package FP::AST::Perl::Let {
+    use FP::Predicates;
+    use FP::List;
+    use FP::Ops ":all";
+
+    use FP::Struct [
+        [list_of *FP::AST::Perl::is_var, 'vars'],
+        [*FP::AST::Perl::is_nonnoop_expr, 'expr'],
+        [*FP::AST::Perl::is_expr, 'body'],
+        ] => "FP::AST::Perl::Expr";
+
+    method string () {
+        my $vars= $self->vars;
+        my $multiple= $vars->length > 1;
+        "my "
+            . ($multiple ? "(" : "")
+            . $vars->map(the_method "string")->strings_join(", ")
+            . ($multiple ? ")" : "")
+            . " = "
+            . $self->expr->string
+            . "; " # XX like Semicolon! But not the same *?* Q
+            . $self->body->string
+    }
+    _END_
 }
 
 
@@ -372,7 +461,12 @@ TEST_EXCEPTION {
     LexVar('foo::bar')
 } 'unacceptable value for field \'name\': \'foo::bar\'';
 
+sub String;
+sub Literal;
+sub PackVarArray;
+sub PackVarCode;
 
-
+sub t{
+}
 
 1
